<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use App\Service\GoodsDetailService;
use Exception;
use App\RedisModel;
use App\Support\EmailQueue;
use App\Models\Goods;
use App\Log as AppLog;

class SyncGoodsDetail extends Command
{
    protected $signature = 'sync:goods-detail';
    protected $description = '同步商品详情数据';
    
    private $startTime;
    const MAX_EXECUTION_TIME = 600; // 10分钟超时时间（秒）

    public function __construct()
    {
        parent::__construct();
        $this->startTime = time();
    }

    /**
     * 执行同步商品详情
     */
    public function handle()
    {
        try {
            $this->info('开始同步商品详情...');
            
            while (true) {
                if ($this->shouldExit()) {
                    break;
                }
                
                $productInfo = $this->getNextProduct();
                if (!$productInfo) {
                    sleep(1);
                    continue;
                }

                GoodsDetailService::processGoodsDetail(
                    $productInfo['shop_appid'],
                    $productInfo['product_id'],
                    $productInfo['plan_type']
                );
            }
            
        } catch (Exception $e) {
            $this->error('同步失败：' . $e->getMessage());
            $errorParams = ['file' => $e->getFile(), 'line' => $e->getLine(), 'error' => $e->getMessage(), 'trace' => $e->getTraceAsString()];
            AppLog::logError('商品详情同步失败', $errorParams, 'sync_goods_detail'); 
            EmailQueue::rPush('商品详情同步失败', json_encode($errorParams, 256), ['song.shen@kuxuan-inc.com'], '微店推客带货');
        }
    }

    // 检查是否超时
    private function shouldExit(): bool
    {
        if (time() - $this->startTime >= self::MAX_EXECUTION_TIME) {
            $this->warn('脚本已运行10分钟，自动退出');
            return true;
        }
        return false;
    }

    /**
     * 获取下一个商品
     */
    private function getNextProduct()
    {
        $productInfo = RedisModel::rPop(Goods::QUEUE_KEY); 
        if (!$productInfo) {
            return null;
        }
        return json_decode($productInfo, true);
    }
} 