<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Service\GoodsService;
use Illuminate\Http\Request;

class GoodsController extends Controller
{
    /**
     * 获取商品列表
     */
    public function list(Request $request)
    {
        $validator = \Validator::make($request->all(),[
          'keyword' => 'nullable',
          'sort' => 'nullable|in:0,1,2,3,4',// 0综合 1销量倒序 2销量正序 3价格正序 4价格倒序
          'page' => 'required|int',
          'page_size' => 'required|int',
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 1102);
        }

        $params['keyword'] = $request->input('keyword');
        $params['sort'] = $request->input('sort', 0);
        $params['page'] = $request->input('page');
        $params['page_size'] = $request->input('page_size');
        $params['user_id'] = $request->input('user_id') ?? 0;

        list($data, $total) = GoodsService::getGoodsList($params);

        return self::returnPageValue($data, $total, $params['page_size'], $params['page']);
    }

    /**
     * 获取商品详情
     */
    public function detail(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'product_id' => 'required|string',
            'sharer_open_id' => 'nullable|string',
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 1102);
        }

        $productId = $request->input('product_id');
        $sharerOpenId = $request->input('sharer_open_id');

        $data = GoodsService::getGoodsDetail($productId, $sharerOpenId);

        return self::returnValue($data);
    }
}   