<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Log as AppLog;
use App\Service\OrderService;
use App\Support\EmailQueue;
use Illuminate\Http\Request;
use Exception;

class OrderController extends Controller
{
    /**
     * 推广订单列表
     */
    public function list(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'page' => 'nullable|integer|min:1',
            'page_size' => 'nullable|integer|min:1|max:50',
            'commission_status' => 'nullable|integer|in:20,100,200',
            'order_status' => 'nullable|integer|in:10,20,21,30,100,200,250',
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 1102);
        }

        $page = $request->input('page', 1);
        $pageSize = $request->input('page_size', 20);
        try {
            $commissionStatus = $request->input('commission_status');
            $orderStatus = $request->input('order_status');

            list($data, $count) = OrderService::getOrderList(
                $request->user()->id,
                $page,
                $pageSize,
                $commissionStatus,
                $orderStatus
            );

            return self::returnPageValue(
                $data,
                $count,
                $pageSize,
                $page
            );
            
        } catch (Exception $e) {
            AppLog::logError('获取订单列表接口失败', [
                'user_id' => $request->user()->id,
                'params' => $request->all(),
                'file' => $e->getFile(),
                'line' => $e->getLine(),
                'message' => $e->getMessage(),
                'trace' => $e->getTraceAsString(),
            ], 'orderListApi');
            
            EmailQueue::rPush('获取订单列表接口失败', json_encode([
                'user_id' => $request->user()->id,
                'params' => $request->all(),
                'file' => $e->getFile(),
                'line' => $e->getLine(),
                'message' => $e->getMessage(),
                'trace' => $e->getTraceAsString(),
            ]), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            
            return self::returnPageValue([], 0, $pageSize, $page);
        }
    }
}