<?php
namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Log as AppLog;
use App\Service\OrderService;
use App\Support\EmailQueue;
use Exception;
use Illuminate\Http\Request;

class UserController extends Controller
{
    // 获取用户收益统计
    public function myEarnings(Request $request) {
        $validator = \Validator::make($request->all(), [
            'user_id' => 'required|integer',
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 1102);
        }

        $userId=$request->input('user_id');

        try {
            $moneyData=OrderService::estimatesOfIncome($userId);//获取收益
            //账户余额（可提现）
            $userInfo['account_balance']=OrderService::changeGoodsBranchToElement(OrderService::accountPrice($userId,2)-OrderService::accountPrice($userId,0));
            
            //今日预估收入
            $userInfo['this_day_forecast_income']  = $moneyData['thisDayIncomePredict'];
            
            //本月预估收入
            $userInfo['this_month_forecast_income']  = $moneyData['thisIncomePredict'];
            
            //本月结算收入
            $userInfo['this_month_settlement_income']= $moneyData['thisMonthSettledCommission'];

            // 今日结算收入
            $userInfo['this_day_settlement_income'] = $moneyData['thisDaySettledCommission'];
           
            //累计结算
            $userInfo['all_settlement_income']= $moneyData['totalSettledCommission'];

            // 待结算收入
            $userInfo['wait_settlement_income'] = $moneyData['waitSettledCommission'];
            
            return self::returnValue($userInfo, 0);
        } catch (Exception $e) {
            $exceptionParams = [
                'params' => $request->all(),
                'file' => $e->getFile(),
                'line' => $e->getLine(),
                'message' => $e->getMessage(),
                'trace' => $e->getTraceAsString(),
            ];

            AppLog::logError('获取用户收益接口请求失败', $exceptionParams, 'myEarnings');
            EmailQueue::rPush('获取用户收益接口请求失败', json_encode($exceptionParams, 256), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return self::returnValue($e->getMessage(), 500);
        }
    }

    // 用户收益提现明细
    public function myEarningsDetail(Request $request) {
        $validator = \Validator::make($request->all(), [
            'user_id' => 'required|string',
            'type' => 'nullable|in:1,2'
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 1102);
        }

        try {
            $userId=$request->input('user_id');

            $returnData = OrderService::myEarningsDetail($userId, $type);

            return self::returnValue($returnData, 0);

        } catch (Exception $e) {
            $exceptionParams = [
                'params' => $request->all(),
                'file' => $e->getFile(),
                'line' => $e->getLine(),
                'message' => $e->getMessage(),
                'trace' => $e->getTraceAsString(),
            ];

            AppLog::logError('获取用户收益接口请求失败', $exceptionParams, 'myEarnings');
            EmailQueue::rPush('获取用户收益接口请求失败', json_encode($exceptionParams, 256), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return self::returnValue($e->getMessage(), 500);
        }
    }
}