<?php
/**
 * Created by PhpStorm.
 * User: Jason
 * Date: 17/3/12
 * Time: 下午11:09
 */
namespace App\Http\Middleware;

use App\Models\AuthorizeCorp;
use App\Models\OfficialAccount;
use App\Models\System\AdminManageCorp;
use App\Models\System\Users;
use Illuminate\Http\Response;
use Illuminate\Http\Request;
use Closure;
use App\RedisModel;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;

class ApiLoginMiddleware
{
    public function handle(Request $request, Closure $next)
    {
        $userId = $request->input('admin_id');
        $ttl = $request->input('ttl');
        $sign = $request->input('sign');
        $sysGroupId = $request->input('sys_group_id');

        $redisKey = env('LOGIN_REDIS_KEY') . '-'. $userId;
        $randomKey = RedisModel::get($redisKey);
        if(!$randomKey){
            $data = Controller::returnValue(['data' => '请重新登录'],4002, '登录失败，请重新登录');
            return new Response($data);
        }

        // 用户
        $user = Users::query()->find($userId);
        // 用户不存在或被删除，退回重新登录
        if (empty($user)) {
            $data = Controller::returnValue(['data' => '请重新登录'],4002, '登录失败，请重新登录');
            return new Response($data);
        }

        if(Users::validSign($userId, $ttl, $randomKey, $sign) === false){
            $data = Controller::returnValue(['data' => '登录失败，请重新登录'],4002, '登录失败，请重新登录');
            return new Response($data);
        }

        $sysGroupId = Users::where('enable', 1)
            ->where(function ($query) use($user, $userId, $sysGroupId) {
                if($user->is_system_admin) {
                    $query->where('group_admin_id', $sysGroupId);
                } else {
                    $query->where('id', $userId);
                }
            })
            ->value('group_admin_id');

        /**登陆账号所属账号组和账号身份**/
        $request->merge([
            'sys_group_id' => $sysGroupId,
            'group_admin_id'=>$user->group_admin_id,
            'is_system_admin'=>$user->is_system_admin,
        ]);

        /**当前登陆账号可以看到的公众号的集合**/
        /**逻辑暂时隐藏，不用开发
        $request->merge([
            'account_manage_mp_app_id_arr' => $this->manage_mp_app_id_list(
                $user->is_system_admin, $user->group_admin_id, $request->input('chose_group_admin_id')
            )
        ]);
         **/
        Auth::setUser($user);

        return $next($request);
    }


    /**
     * 获取管理的公众号集合
     * @param $is_system_admin [是否是系统管理员 1是 0否]
     * @param $current_group_admin_id [账号当前所在分组ID]
     * @param $chose_group_admin_id [是否手动选择账号分组（系统管理员支持筛选）]
     */
    private function manage_mp_app_id_list($is_system_admin,$current_group_admin_id,$chose_group_admin_id){

        $search_group_admin_id = [];    //默认为空，查看不限制
        /**系统管理员查看全部或筛选查看分组**/
        if($is_system_admin == Users::SYSTEM_ADMIN){
            if(!empty($chose_group_admin_id)){
                $search_group_admin_id[] = $chose_group_admin_id;
            }
        }else{
        /**非系统管理员无筛选权利，只查看自己的分组内容**/
            $search_group_admin_id[] = $current_group_admin_id;
        }



        /**查询管理的公众号**/
        $query_mp_app_id = OfficialAccount::query()->where("enable",1);

        if(!empty($search_group_admin_id)) {
            /***
             * 查看分组对应管理的主体列表
             */
            $manage_corp_id_arr = AdminManageCorp::query()
                ->where("is_delete",0)
                ->whereIn("sys_user_id",$search_group_admin_id)
                ->pluck("corpid")
                ->toArray();
            /***
             * 根据主体集合获取对应公众号集合
             */
            $corpid_str_arr = AuthorizeCorp::query()->where("enable",1)
                ->whereIn("id",$manage_corp_id_arr)
                ->pluck("corpid")->toArray();

            $query_mp_app_id->whereIn("corp_id",$corpid_str_arr);
        }
        return $query_mp_app_id->pluck("mp_app_id")->toArray();
    }
}