<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;

class AccountDetail extends Model
{
    /**
     * The table associated with the model.
     *
     * @var string
     */
    protected $table = 'account_detail';
    public $timestamps = false; 
    protected static $unguarded = true;

    /**
     * The attributes that should be cast.
     *
     * @var array
     */
    protected $casts = [
        'id' => 'integer',
        'user_id' => 'integer',
        'type' => 'integer',
        'rebate' => 'integer',
        'state_of_embodiment' => 'integer',
        'enable' => 'integer',
        'create_at' => 'datetime',
        'update_at' => 'datetime'
    ];

    /**
     * 类型声明
     */
    const TYPE_COMMISSION = 1;  // 佣金
    const TYPE_WITHDRAWAL = 2;  // 提现

    /**
     * 状态声明
     */
    const STATE_PENDING = 0;    // 申请中
    const STATE_APPROVED = 1;   // 已经通过
    const STATE_CANCELLED = 2;  // 已经废除

    /**
     * 数据状态
     */
    const STATUS_ENABLED = 1;   // 生效

    // 获取用户收入
    public static function getIncomeByUserId($userId) {
        return self::query()->where('enable', 1)->where("user_id",$userId)->where("type", self::TYPE_COMMISSION)->sum("rebate");
    }

    // 获取用户提现
    public static function getWithdrawalByUserId($userId) {
        return self::query()->where('enable', 1)->where("user_id",$userId)->where("type", self::TYPE_WITHDRAWAL)
        ->whereIn("state_of_embodiment", [self::STATE_APPROVED, self::STATE_PENDING])->sum("rebate");
    }

    // 获取用户已提现
    public static function getWithdrawalByUserIdApproved($userId) {
        return self::query()->where('enable', 1)->where("user_id",$userId)->where("type", self::TYPE_WITHDRAWAL)
        ->where("state_of_embodiment", self::STATE_APPROVED)->sum("rebate");
    }

    public static function getMyEarningsDetail($userId) {
        return self::query()->where('enable', 1)->where("user_id",$userId)->where('state_of_embodiment', '!=', self::STATE_CANCELLED)->orderBy('create_at', 'desc')->get();
    }
}
