<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;

class Goods extends Model
{
    protected $table = 'wechat_goods';
    protected $primaryKey = 'id';
    public $timestamps = false;
    protected static $unguarded = true;
    
    const CREATED_AT = 'create_time';
    const UPDATED_AT = 'update_time';
    const QUEUE_KEY = 'wechatStore:goods_detail_queue';
    const FAILED_QUEUE_KEY = 'wechatStore:goods_detail_failed_queue';// 要实际跑起来才知道有哪些情况是需要放入重试队列中的

    /**
     * 获取该商品的所有SKU
     */
    public function skus()
    {
        return $this->hasMany(GoodsSkus::class, 'product_id', 'product_id');
    }

    /**
     * 保存前的处理
     */
    protected static function boot()
    {
        parent::boot();
        
        static::saving(function ($model) {
            // 确保图片列表不超过限制
            if (is_array($model->head_imgs) && count($model->head_imgs) > 9) {
                $model->head_imgs = array_slice($model->head_imgs, 0, 9);
            }
            if (is_array($model->desc_imgs) && count($model->desc_imgs) > 20) {
                $model->desc_imgs = array_slice($model->desc_imgs, 0, 20);
            }
        });
    }

    /**
     * 保存商品详情数据
     */
    public static function saveGoodsDetail($appId, $productId, $planType, $detail)
    {
        return self::updateOrCreate(
            [
                'shop_appid' => $appId,
                'product_id' => $productId
            ],
            [
                'title' => $detail['product_info']['title'] ?? '',
                'sub_title' => $detail['product_info']['sub_title'] ?? '',
                'head_imgs' => !empty($detail['product_info']['head_imgs']) ? json_encode($detail['product_info']['head_imgs'], 256) : '',
                'desc_imgs' => !empty($detail['product_info']['desc_info']['desc_imgs']) ? json_encode($detail['product_info']['desc_info']['desc_imgs'], 256) : '',
                'desc_text' => $detail['product_info']['desc_info']['desc'] ?? '',
//                'cats' => !empty($detail['product_info']['cats']) ? implode(',', $detail['product_info']['cats']) : '',
                'cats_v2' => $detail['product_info']['cats_v2'] ?? '',
                'commission_info' => !empty($detail['commission_info']) ? json_encode($detail['commission_info'], 256) : '',
                'item_status' => $detail['commission_info']['status'] ?? 0,
                'min_sale_price' => !empty($detail['product_info']['min_sale_price']) ? round($detail['product_info']['min_sale_price'] / 100, 2) : 0,
                'max_sale_price' => !empty($detail['product_info']['max_sale_price']) ? round($detail['product_info']['max_sale_price'] / 100, 2) : 0,
                'min_stock_num' => $detail['product_info']['min_stock_num'] ?? 0,
                'max_stock_num' => $detail['product_info']['max_stock_num'] ?? 0,
                'enable' => 1,
                'plan_type' => $planType,
                'update_status' => 1,
                'product_promotion_link' => $detail['product_promotion_link'],
                'normal_commission_ratio' => !empty($detail['commission_info']['normal_commission_info']['ratio']) ? round($detail['commission_info']['normal_commission_info']['ratio']/10000, 2) : 0,
            ]
        );
    }

    public static function getGoodsListData($params)
    {
        $query = Goods::query()->where('enable', 1)->where('item_status', 1);

        // 关键词搜索
        if (!empty($params['keyword'])) {
            $query->where(function ($query) use ($params) {
                $query->where('title', 'like', "%{$params['keyword']}%")
                    ->orWhere('sub_title', 'like', "%{$params['keyword']}%");
            });
        }

        $count = $query->count();
        $query->select(['id', 'title', 'sub_title', 'head_imgs', 'product_id', 'min_sale_price', 'max_sale_price', 'min_stock_num', 'max_stock_num'])
            ->limit($params['page_size']);
        switch ($params['sort']) {
            case 1:
                $query->orderBy('max_stock_num', 'desc');         
                break;
            case 2:
                $query->orderBy('min_stock_num', 'asc');
                break;
            case 3:
                $query->orderBy('min_sale_price', 'asc');
                break;
            case 4:
                $query->orderBy('max_sale_price', 'desc');
                break;
            default:
                $query->orderBy('create_time', 'desc');
                break;
        }
        $data = $query->offset(($params['page'] - 1) * $params['page_size'])
            ->get();

        return [$data, $count];
    }

    public static function getGoodsDetail($productId)
    {
        return Goods::with(['skus' => function($query) {
            $query->where('enable', 1)->orderBy('stock_num', 'desc');
        }])
        ->where('product_id', $productId)
        ->where('enable', 1)
        ->firstOrFail();
    }

    public static function resetStatus(){
        Goods::query()->where('enable', 1)->where('update_status', 0)->update(['enable' => 0]);
        Goods::query()->where('enable', 1)->update(['update_status' => 0]);
    }
}