<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;

class GoodsSkus extends Model
{
    protected $table = 'wechat_goods_skus'; // 表名 
    protected $primaryKey = 'id'; // 主键
    public $timestamps = false; // 不自动维护时间戳
    protected static $unguarded = true;
    
    const CREATED_AT = 'create_time'; // 创建时间
    const UPDATED_AT = 'update_time'; // 更新时间
    const GOODS_DETAIL_QUEUE = 'wechatStore:goods_detail_queue'; // 商品详情队列
    const GOODS_DETAIL_FAILED_QUEUE = 'wechatStore:goods_detail_failed_queue'; // 商品详情失败队列

    /**
     * 批量保存商品SKU数据
     */
    public static function saveGoodsSkus($appId, $productId, $skus)
    {
        // 先将该商品的所有SKU标记为无效
        self::where([
            'shop_appid' => $appId,
            'product_id' => $productId
        ])->update(['enable' => 0]);

        // 更新或创建新的SKU
        foreach ($skus as $sku) {
            self::updateOrCreate(
                [
                    'shop_appid' => $appId,
                    'product_id' => $productId,
                    'sku_id' => $sku['sku_id']
                ],
                [
                    'thumb_img' => $sku['thumb_img'] ?? '',
                    'stock_num' => $sku['stock_num'] ?? 0,
                    'sale_price' => !empty($sku['sale_price']) ? round($sku['sale_price'] / 100 , 2) : 0,
                    'attrs' => !empty($sku['sku_attrs']) ? json_encode($sku['sku_attrs'], 256) : '',
                    'enable' => 1
                ]
            );
        }
    }

    public static function getSkusByProductId($appId, $productIdList)
    {
        return self::where('app_id', $appId)->whereIn('product_id', $productIdList)->where('enable', 1)->get();
    }
}