<?php

namespace App\Service;

use App\Log as AppLog;
use App\Models\Goods;
use App\Models\GoodsSkus;
use App\Support\EmailQueue;
use App\Support\weChatApi;
use Exception;

class GoodsDetailService
{
    /**
     * 处理商品详情数据
     */
    public static function processGoodsDetail($shopAppId, $productId, $planType): bool
    {
        try {
            // 获取商品详情
            $detail = weChatApi::getPromoteGoodsInfo($shopAppId, $productId, $planType);
            if (empty($detail) || $detail['errcode'] != 0) {
                $errorParams = ['appId' => $shopAppId, 'productId' => $productId, 'error' => '获取商品详情失败', 'detail' => $detail];
                EmailQueue::rPush('获取商品详情失败', json_encode($errorParams), ['song.shen@kuxuan-inc.com'], '微店推客带货');  
                AppLog::logError('获取商品详情失败', $errorParams, 'sync_goods_detail');    
                return false;
            }

            // 解析sku内容，提取最大库存和最低价格
            if (!empty($detail['product']['product_info']['skus'])) {
                $minStock = min(array_column($detail['product']['product_info']['skus'], 'stock_num'));
                $maxSalePrice = max(array_column($detail['product']['product_info']['skus'], 'sale_price'));
                $maxStock = max(array_column($detail['product']['product_info']['skus'], 'stock_num'));
                $minSalePrice = min(array_column($detail['product']['product_info']['skus'], 'sale_price'));
                $detail['product']['product_info']['min_stock_num'] = $minStock;
                $detail['product']['product_info']['max_sale_price'] = $maxSalePrice;
                $detail['product']['product_info']['max_stock_num'] = $maxStock;
                $detail['product']['product_info']['min_sale_price'] = $minSalePrice;
            }

            if(!empty($detail['product']['product_info']['cats_v2'])) {
                $detail['product']['product_info']['cats_v2'] = implode(',', array_map(function($value){
                    return intval($value['cat_id']);
                }, $detail['product']['product_info']['cats_v2']));
            }

            \DB::beginTransaction();
            
            // 保存商品基础信息
            Goods::saveGoodsDetail($shopAppId, $productId, $planType,$detail['product']);

            // 保存SKU信息
            if (!empty($detail['product']['product_info']['skus'])) {
                GoodsSkus::saveGoodsSkus($shopAppId, $productId, $detail['product']['product_info']['skus']);
            }
            
            \DB::commit();
            return true;

        } catch (Exception $e) {
            \DB::rollBack();
            $errorParams = ['appid' => $shopAppId, 'productId' => $productId,  'file' => $e->getFile(), 'line' => $e->getLine(), 'error' => $e->getMessage(), 'trace' => $e->getTraceAsString()];
            EmailQueue::rPush('处理商品详情失败', json_encode($errorParams, 256), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            AppLog::logError('处理商品详情失败', $errorParams, 'sync_goods_detail');  
            return false;
        }
    }
} 