<?php

namespace App\Service;

use App\Log as AppLog;
use App\Models\Goods;
use App\RedisModel;
use App\Support\EmailQueue;
use App\Support\weChatApi;
use Exception;

class GoodsListService
{
    const BATCH_SIZE = 30;

    /**
     * 处理商品列表数据
     */
    public static function processGoodsList($planType, $spuItemCondition, $category, $keyword, $nextKey = ''): array
    {
        try {
            // 调用微信API获取商品列表
            $response = weChatApi::getPromoteGoodsList($planType, $spuItemCondition, $category, $keyword, $nextKey, self::BATCH_SIZE);

            if (empty($response) || $response['errcode'] != 0 || empty($response['product_list'])) {
                return [
                    'success' => false,
                    'next_key' => $nextKey,
                    'count' => 0
                ];
            }

            // 将商品ID存入Redis队列
            foreach ($response['product_list'] as $good) {
                RedisModel::rPush(Goods::QUEUE_KEY, json_encode([
                    'shop_appid' => $good['shop_appid'],
                    'product_id' => $good['product_id'],
                    'plan_type' => $planType
                ], 256));
            }

            return [
                'success' => true,
                'next_key' => $response['next_key'] ?? $nextKey,
                'count' => count($response['product_list'])
            ];

        } catch (Exception $e) {
            $errorParams = ['file' => $e->getFile(), 'line' => $e->getLine(), 'error' => $e->getMessage(), 'trace' => $e->getTraceAsString()];
            AppLog::logError('获取商品列表失败', $errorParams, 'sync_goods_list');
            EmailQueue::rPush('获取商品列表失败', json_encode($errorParams), ['song.shen@kuxuan-inc.com'], '微店推客带货');

            return [
              'success' => false,
              'next_key' => $nextKey,
              'count' => 0
          ];
        }
    }
} 