<?php

namespace App\Support;

use App\Log;
use App\Service\HttpService;

class weChatApi
{
    // 获取稳定版access_token
    public static function getStableToken($appId, $appSecret, $forceRefresh = false) {
        $url = config('wechat_api.stable_token');

        $params = [
            'grant_type' => 'client_credential',
            'app_id' => $appId,
            'secret' => $appSecret,
            'force_refresh' => $forceRefresh
        ];

        $response = HttpService::httpPost($url, $params);
        if(empty($response) || (isset(json_decode($response, true)['errcode']) && json_decode($response, true)['errcode'] == -1)) { 
            Log::logError('获取稳定版access_token结果为空', $params, 'weChatApi');
            EmailQueue::rPush('获取稳定版access_token结果为空', json_encode($params), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return false;
        }
        $response = json_decode($response, true);

        if( !isset($response['errcode']) || $response['errcode'] !== 0 ){
            Log::logError('获取稳定版access_token失败', ['params' => $params, 'response' => $response], 'weChatApi');
            EmailQueue::rPush('获取稳定版access_token失败', json_encode(['params' => $params, 'response' => $response]), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return false;
        }

        return $response;
    }

    // 获取招商团长access_token
    public static function getAccessToken($appId, $appSecret) {
        $url = config('wechat_api.access_token');

        $params = [
            'grant_type' => 'client_credential',
            'app_id' => $appId,
            'secret' => $appSecret,
        ];
        $url = $url.'?'.http_build_query($params);

        $response = HttpService::httpGet($url);
        if(empty($response) || (isset(json_decode($response, true)['errcode']) && json_decode($response, true)['errcode'] == -1)) { 
            Log::logError('获取招商团长access_token结果为空', $params, 'weChatApi');
            EmailQueue::rPush('获取招商团长access_token结果为空', json_encode($params), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return false;
        }
        $response = json_decode($response, true);

        if( !isset($response['errcode']) || $response['errcode'] !== 0 ){
            Log::logError('获取招商团长access_token失败', ['params' => $params, 'response' => $response], 'weChatApi');
            EmailQueue::rPush('获取招商团长access_token失败', json_encode(['params' => $params, 'response' => $response]), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return false;
        }

        return $response;
    }

    // 获取机构合作商品列表
    public static function getGoodsList($appId, $pageSize, $nextKey, $retry = 0) {
        $accessToken = '';
        $url = config('wechat_api.goods_list').$accessToken;

        $params = [
            'appid' => $appId,
            'page_size' => $pageSize,
            'next_key' => $nextKey
        ];

        $response = HttpService::httpPost($url, $params);
        if(empty($response) || (isset(json_decode($response, true)['errcode']) && json_decode($response, true)['errcode'] == -1)) { 
            if($retry < 3) {
                $retry++;
                sleep(1);
                return self::getGoodsList($appId, $pageSize, $nextKey, $retry);
            }
            Log::logError('获取机构合作商品列表结果为空', $params, 'weChatApi');
            EmailQueue::rPush('获取机构合作商品列表结果为空', json_encode($params), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return false;
        }
        $response = json_decode($response, true);

        if( !isset($response['errcode']) || $response['errcode'] !== 0 ){
            Log::logError('获取机构合作商品列表失败', ['params' => $params, 'response' => $response], 'weChatApi');
            EmailQueue::rPush('获取机构合作商品列表失败', json_encode(['params' => $params, 'response' => $response]), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return false;
        }

        return $response;
    }

    // 获取机构合作商品详情
    public static function getGoodsInfo($appId, $productId, $retry = 0) {
        $accessToken = '';
        $url = config('wechat_api.goods_info').$accessToken;

        $params = [
            'appid' => $appId,
            'product_id' => $productId
        ];

        $response = HttpService::httpPost($url, $params);
        if(empty($response) || (isset(json_decode($response, true)['errcode']) && json_decode($response, true)['errcode'] == -1)) { 
            if($retry < 3) {
                $retry++;
                sleep(1);
                return self::getGoodsInfo($appId, $productId, $retry);
            }
            Log::logError('获取机构合作商品详情结果为空', $params, 'weChatApi');
            EmailQueue::rPush('获取机构合作商品详情结果为空', json_encode($params), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return false;
        }
        $response = json_decode($response, true);

        if( !isset($response['errcode']) || $response['errcode'] !== 0 ){
            Log::logError('获取机构合作商品详情失败', ['params' => $params, 'response' => $response], 'weChatApi');
            EmailQueue::rPush('获取机构合作商品详情失败', json_encode(['params' => $params, 'response' => $response]), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return false;
        }

        return $response;
    }

    // 获取推客可推广商品列表
    public static function getPromoteGoodsList($planType, $spuItemCondition, $category, $keyword, $nextKey, $pageSize, $retry = 0) {
        $testData = '{"errcode": "0","errmsg": "ok","product_list":[{"product_id": 12345,"shop_appid": "test"}],"next_key": ""}';
        return json_decode($testData, true);

        // todo
        $accessToken = '';

        $url = config('wechat_api.promote_goods_list').$accessToken;

        $params = [
            'plan_type' => $planType,
            'next_key' => $nextKey,
            'page_size' => $pageSize
        ];

        if(!empty($spuItemCondition)) {
            $params['spu_item_condition'] = $spuItemCondition;
        }
        if(!empty($category)) {
            $params['category'] = $category;
        }
        if(!empty($keyword)) {
            $params['keyword'] = $keyword;
        }

        $response = HttpService::httpPost($url, $params);
        if(empty($response) || (isset(json_decode($response, true)['errcode']) && json_decode($response, true)['errcode'] == -1)) { 
            if($retry < 3) {
                $retry++;
                sleep(1);
                return self::getPromoteGoodsList($planType, $spuItemCondition, $category, $keyword, $nextKey, $pageSize, $retry);
            }
            Log::logError('获取推客可推广商品列表结果为空', $params, 'weChatApi');
            EmailQueue::rPush('获取推客可推广商品列表结果为空', json_encode($params, 256), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return false;
        }
        $response = json_decode($response, true);

        if( !isset($response['errcode']) || $response['errcode'] !== 0 ){
            Log::logError('获取推客可推广商品列表失败', ['params' => $params, 'response' => $response], 'weChatApi');
            EmailQueue::rPush('获取推客可推广商品列表失败', json_encode(['params' => $params, 'response' => $response], 256), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return false;
        }

        return $response;
    }

    // 获取推客可推广商品详情
    public static function getPromoteGoodsInfo($shopAppId, $productId, $planType, $retry = 0) {
        $testData = '{
	"errcode": "0",
	"errmsg": "ok",
	"product": {
		"shop_appid": "test",
		"product_id": 12345,
		"product_promotion_link": "",
		"product_info": {
			"title": "test_title",
			"sub_title": "",
			"head_imgs": [
				"https://test.com/0"
			],
			"desc_info": {
				"imgs": [
					"https://test.com/0"
				],
				"desc": ""
			},
			"cats_v2": [{
				"cat_id": "1421"
			}],
			"express_info": {
				"send_time": "",
				"address_info": {
					"postal_code": "",
					"province_name": "",
					"city_name": "",
					"county_name": ""
				},
				"shipping_method": "NO_FREE"
			},
			"extra_service": {
				"seven_day_return": 0,
				"pay_after_use": 0,
				"freight_insurance": 0,
				"fake_one_pay_three": 0,
				"damage_guarantee": 0
			},
			"statistical_data": {
				"good_ratio": 0,
				"sales_in_30_days": 1
			},
			"skus": [{
				"sku_id": "670813472",
				"thumb_img": "",
				"sale_price": 500,
				"stock_num": 998,
				"sku_attrs": [{
					"attr_key": "产地",
					"attr_value": "四川成都"
				}]
			}]
		},
		"commission_info": {
			"status": 1,
			"service_ratio": 1,
			"start_time": 1,
			"end_time": 1,
			"normal_commission_info": {
				"ratio": 1
			}
		}
	}
}';

        return json_decode($testData, true);

        $accessToken = '';

        $url = config('wechat_api.promote_goods_info').$accessToken;

        $params = [
            'shop_appid' => $shopAppId,
            'product_id' => $productId,
            'plan_type' => $planType
        ];

        $response = HttpService::httpPost($url, $params);
        if(empty($response) || (isset(json_decode($response, true)['errcode']) && json_decode($response, true)['errcode'] == -1)) {
            if($retry < 3) {
                $retry++;
                sleep(1);
                return self::getPromoteGoodsInfo($shopAppId, $productId, $planType, $retry);
            }
            Log::logError('获取推客可推广商品详情结果为空', $params, 'weChatApi');
            EmailQueue::rPush('获取推客可推广商品详情结果为空', json_encode($params), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return false;
        }
        $response = json_decode($response, true);

        if( !isset($response['errcode']) || $response['errcode'] !== 0 ){
            Log::logError('获取推客可推广商品详情失败', ['params' => $params, 'response' => $response], 'weChatApi');
            EmailQueue::rPush('获取推客可推广商品详情失败', json_encode(['params' => $params, 'response' => $response]), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return false;
        }

        return $response;
    }

    /**
     * 获取推客的注册状态，以及和机构的绑定状态
     * @param $shareOpenid string 推客在小程序中的 openid，和 $shareAppid 二选一
     * @param $shareAppid string 推客在微信电商平台注册的身份标识，和 $shareOpenid 二选一
     * @param $isSimpleRegister bool 是否走简易版本注册【当走简易版本注册时，可以不要求推客开通商户号，但分佣只能走机构自己分佣；如果不走简易注册流程时，要求推客开通商户号作为收款账户，可以平台分佣】
     * */
    public static function getPromoterRegAndBindStatus($shareOpenid, $shareAppid, $isSimpleRegister)
    {
        $accessToken = '';
        $url = config('wechat_api.get_promoter_register_and_bind_status').$accessToken;
        $params = [
            'share_openid' => $shareOpenid,
            'share_appid' => $shareAppid,
            'is_simple_register' => $isSimpleRegister
        ];
        $response = HttpService::httpPost($url, $params);
        $response = json_decode($response, true);

        if($response === false || (isset($response['errcode']) && $response['errcode'] !== 0)){
            Log::logError('获取推客注册状态失败', ['params' => $params, 'response' => $response], 'weChatApi');
            EmailQueue::rPush('获取推客注册状态失败', json_encode(['params' => $params, 'response' => $response]), ['xiaohua.hou@kuxuan-inc.com'], '微店推客带货');
            return false;
        }

        return $response;
    }

    /**
     * 获取已经绑定的推客信息
     * @param $accessToken string
     * */
    public static function getPromoterList($shareOpenid, $shareAppid, $nextKey, $pageSize)
    {
        $accessToken = '';
        $url = config('wechat_api.get_promoter_list').$accessToken;
        $params = [
            'share_openid' => $shareOpenid,
            'share_appid' => $shareAppid,
            'next_key' => $nextKey,
            'page_size' => $pageSize
        ];
        $response = HttpService::httpPost($url, $params);
        $response = json_decode($response, true);

        if($response === false || (isset($response['errcode']) && $response['errcode'] !== 0)){
            Log::logError('获取推客列表失败', ['params' => $params, 'response' => $response], 'weChatApi');
            EmailQueue::rPush('获取推客列表失败', json_encode(['params' => $params, 'response' => $response]), ['xiaohua.hou@kuxuan-inc.com'], '微店推客带货');
            return false;
        }

        return $response;
    }

    /**
     * 获取小程序用户信息
     */
    public static function getUserInfo($code)
    {
        $url = config('wechat_api.code2session');
        $params = [
            'appid' => env('WECHAT_APPID'),
            'secret' => env('WECHAT_APPSECRET'),
            'js_code' => $code,
            'grant_type' => 'authorization_code'
        ];
        $response = HttpService::httpGet($url . http_build_query($params));
        $response = json_decode($response, true);

        if($response === false || (isset($response['errcode']) && $response['errcode'] !== 0)){
            Log::logError('获取小程序用户信息失败', ['params' => $params, 'response' => $response], 'weChatApi');
            EmailQueue::rPush('获取小程序用户信息失败', json_encode(['params' => $params, 'response' => $response]), ['xiaohua.hou@kuxuan-inc.com'], '微店推客带货');
            return false;
        }

        return $response;
    }

    // 获取佣金单列表
    public static function getOrderList($appId, $finderId, $openFinderId, $createTimeRange, $updateTimeRange, $orderId, $pageSize, $nextKey, $retry = 0) {
        // todo
        $accessToken = '';

        $url = config('wechat_api.order_list').$accessToken;

        $params = [
            'appid' => $appId,
            'page_size' => $pageSize,
            'next_key' => $nextKey
        ];

        if(!empty($finderId)) {
            $params['finder_id'] = $finderId;
        }
        if(!empty($openFinderId)) {
            $params['open_finder_id'] = $openFinderId;
        }
        if(!empty($orderId)) {
            $params['order_id'] = $orderId;
        }
        if(!empty($createTimeRange)) {
            $params['create_time_range'] = $createTimeRange;
        }
        if(!empty($updateTimeRange)) {
            $params['update_time_range'] = $updateTimeRange;
        }

        $response = HttpService::httpPost($url, $params);
        if(empty($response) || (isset(json_decode($response, true)['errcode']) && json_decode($response, true)['errcode'] == -1)) { 
            if($retry < 3) {
                $retry++;
                sleep(1);
                return self::getOrderList($appId, $finderId, $openFinderId, $createTimeRange, $updateTimeRange, $orderId, $pageSize, $nextKey, $retry);
            }
            Log::logError('获取佣金单列表结果为空', $params, 'weChatApi');
            EmailQueue::rPush('获取佣金单列表结果为空', json_encode($params), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return false;
        }
        $response = json_decode($response, true);

        if( !isset($response['errcode']) || $response['errcode'] !== 0 ){
            Log::logError('获取佣金单列表失败', ['params' => $params, 'response' => $response], 'weChatApi');
            EmailQueue::rPush('获取佣金单列表失败', json_encode(['params' => $params, 'response' => $response]), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return false;
        }

        return $response;   
    }

    // 获取佣金单详情
    public static function getOrderInfo($orderId, $skuId, $retry = 0) {
        // todo
        $accessToken = '';

        $url = config('wechat_api.order_info').$accessToken;

        $params = [
            'order_id' => $orderId,
            'sku_id' => $skuId
        ];

        $response = HttpService::httpPost($url, $params);
        if(empty($response) || (isset(json_decode($response, true)['errcode']) && json_decode($response, true)['errcode'] == -1)) { 
            if($retry < 3) {
                $retry++;
                sleep(1);
                return self::getOrderInfo($orderId, $skuId, $retry);
            }
            Log::logError('获取佣金单详情结果为空', $params, 'weChatApi');
            EmailQueue::rPush('获取佣金单详情结果为空', json_encode($params), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return false;
        }
        $response = json_decode($response, true);

        if( !isset($response['errcode']) || $response['errcode'] !== 0 ){
            Log::logError('获取佣金单详情失败', ['params' => $params, 'response' => $response], 'weChatApi');
            EmailQueue::rPush('获取佣金单详情失败', json_encode(['params' => $params, 'response' => $response]), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return false;
        }

        return $response;
    }

    // 获取推客推广链接
    public static function getPromoteLink($sharerAppId, $sharerOpenId, $productId, $shopAppId, $productShortLink,  $retry = 0) {
        // todo
        $accessToken = ''; 

        $url = config('wechat_api.promote_link').$accessToken;

        $params = [];
        if(!empty($sharerAppId)) {
            $params['sharer_appid'] = $sharerAppId;
        }
        if(!empty($sharerOpenId)) {
            $params['sharer_openid'] = $sharerOpenId;
        }
        if(!empty($productId)) {
            $params['product_id'] = $productId;
        }
        if(!empty($shopAppId)) {
            $params['shop_appid'] = $shopAppId;
        }
        if(!empty($productShortLink)) {
            $params['product_short_link'] = $productShortLink;
        }

        $response = HttpService::httpPost($url, $params);
        if(empty($response) || (isset(json_decode($response, true)['errcode']) && json_decode($response, true)['errcode'] == -1)) { 
            if($retry < 3) {
                $retry++;
                sleep(1);
                return self::getPromoteLink($sharerAppId, $sharerOpenId, $productId, $shopAppId, $productShortLink, $retry);
            }
            Log::logError('获取推客推广链接结果为空', $params, 'weChatApi');
            EmailQueue::rPush('获取推客推广链接结果为空', json_encode($params), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return false;
        }
        $response = json_decode($response, true);

        if( !isset($response['errcode']) || $response['errcode'] !== 0 ){
            Log::logError('获取推客推广链接失败', ['params' => $params, 'response' => $response], 'weChatApi');
            EmailQueue::rPush('获取推客推广链接失败', json_encode(['params' => $params, 'response' => $response]), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return false;
        }

        return $response;
    }

    // 获取设置的推客推广佣金比例
    public static function promoteGoodsCommission($sharerAppId, $productId, $retry = 0) {
        // todo
        $accessToken = '';

        $url = config('wechat_api.promote_goods_commission').$accessToken;

        $params = [
            'sharer_appid' => $sharerAppId,
            'product_id' => $productId
        ];

        $response = HttpService::httpPost($url, $params);
        if(empty($response) || (isset(json_decode($response, true)['errcode']) && json_decode($response, true)['errcode'] == -1)) { 
            if($retry < 3) {
                $retry++;
                sleep(1);
                return self::promoteGoodsCommission($sharerAppId, $productId, $retry);
            }
            Log::logError('获取推客推广佣金比例结果为空', $params, 'weChatApi');
            EmailQueue::rPush('获取推客推广佣金比例结果为空', json_encode($params), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return false;
        }
        $response = json_decode($response, true);

        if( !isset($response['errcode']) || $response['errcode'] !== 0 ){
            Log::logError('获取推客推广佣金比例失败', ['params' => $params, 'response' => $response], 'weChatApi');
            EmailQueue::rPush('获取推客推广佣金比例失败', json_encode(['params' => $params, 'response' => $response]), ['song.shen@kuxuan-inc.com'], '微店推客带货');
            return false;
        }

        return $response;
    }
}
